/*
 * Java CGI Library
 *
 * Copyright (c) Matthew Johnson 2004
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the author, please email src@matthew.ath.cx
 *
 */


package cx.ath.matthew.cgi;

import java.util.Iterator;
import java.util.Vector;


/**
 * Class to manage drawing HTML forms
 */
public class HTMLForm
{
   private String target;
   private String submitlabel;
   private Vector fields;
   
   /**
    * @param target The module to submit to
    * @param submitlabel The string to display on the submit button
    */
   public HTMLForm(String target, String submitlabel)
   {
      this.target = target;
      this.submitlabel = submitlabel;
      fields = new Vector();
   }

   /**
    * Add a field to be displayed in the form.
    *
    * @param field A Field subclass.
    */
   public void addField(Field field)
   {
      fields.add(field);
   }
   
   /**
    * Shows the form.
    * @param cgi The CGI instance that is handling output
    */
   public void display(CGI cgi)
   {
      cgi.out("<form action='"+target+"' method='post'>");
      cgi.out("<table>");
      
      Iterator i = fields.iterator();
      while (i.hasNext()) {
         Field f = (Field) i.next();
         if (! (f instanceof HiddenField)) {
            cgi.out("   <tr>");
            cgi.out("      <td>"+f.label+"</td>");
            cgi.out("      <td>"+f.print()+"</td>");
            cgi.out("   </tr>");
         }
      }
      cgi.out("   <tr>");
      cgi.out("      <td colspan='2' style='text-align:center;'>");
      i = fields.iterator();
      while (i.hasNext()) {
         Field f = (Field) i.next();
         if (f instanceof HiddenField) {
            cgi.out("         "+f.print());
         }
      }      
      cgi.out("         <input type='submit' name='submit' value='"+submitlabel+"' />");
      cgi.out("      </td>");
      cgi.out("   </tr>");
      cgi.out("</table>");
      cgi.out("</form>");
   }
}



