package org.ucam.srcf.assassins;

import java.io.IOException;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;
import org.ucam.srcf.assassins.xml.Config;

public class NewsPage extends PageCreator {

   private int num;
   private String base;
   private String title;

   public NewsPage(String target, Map plugins) throws IOException {
      this(target, plugins, new NewsComparator());
   }

   public NewsPage(String target, Map plugins, Comparator c) throws IOException {
      super(target, plugins, c);
      base = target.substring(0,target.length()-4);
      String t = Config.getConfig("vanilla","news-page-name");
      String x = target.substring(target.indexOf(t)+t.length(), target.length());
      if (Debug.debug) Debug.print(NewsPage.class, x);
      this.num = Integer.parseInt(x.substring(0,2));
      this.title = "News "+(num<10?"0":"")+num;
      //days=new HashMap();
   }

   public void generate() throws AssassinSQLException, StateBuilderException {
      if (Debug.debug) Debug.print(NewsPage.class,"generate() called for "+target);
      if (Debug.debug) Debug.print(NewsPage.class,"output stream: "+out);
      out.println("<?xml version=\"1.0\" encoding=\"UTF-8\" ?>");
      out.println("<!DOCTYPE webpage SYSTEM \"NewsPage.dtd\" >");
      out.println("<webpage>");
      out.println("  <title>Week "+num+" News</title>");
      Iterator iter = events.iterator();//(days.keySet()).iterator();
      out.println("  <events>");
      String lastDay = "";
      while (iter.hasNext()) {
         Event e = (Event) iter.next();
         // Long t = (Long) iter.next();
         // String t = (String) iter.next();
         // StringTokenizer s = new StringTokenizer(t);
         // out.println("    <day>"+s.nextToken()+"</day>");
         String day = Tools.prettyDate(e.getEventTime());
         if (!(lastDay.equals(day)))
         {
            lastDay = day;
            out.println("    <day>"+day+"</day>");
         }
         //Iterator iterator = ((Vector)days.get(t)).iterator();
         //while (iterator.hasNext()) {
            //Event e = (Event) iterator.next();
            Map parttoRepl = new HashMap();
            if (Debug.debug) Debug.print(NewsPage.class,"Generating Event" +e);
            if (Debug.debug) Debug.print(NewsPage.class,"NP01: event " + e.getEid() + " in page " + this.getLinkName());
            out.println("    <event>");
            String thisHeadline = e.getHeadline();
            Map map = e.getParticipants();
            Map mapP = e.getParticipantPseudonyms();
            Iterator mapIt = (map.keySet()).iterator();
            if (Debug.debug) Debug.print(NewsPage.class, "Size: "+map.size());
            while (mapIt.hasNext()) {
               Integer x = (Integer) mapIt.next();
               Pseudonym p = (Pseudonym) mapP.get(x);
               Assassin t2 = ((Assassin) map.get(x));
               Debug.print(NewsPage.class, "Calling updatePseudonymMap with "+x+", "+t2+", "+p);
               parttoRepl = updatePseudonymMap(parttoRepl, x, t2, p, e);
               Debug.print(NewsPage.class, "updatePseudonymMap successful");
            }
            thisHeadline= Tools.replace(thisHeadline, parttoRepl);
            out.println("      <headline>"+thisHeadline+"</headline>");
            out.println("      <anchor>e"+(e.getEid()+Config.getConfigInt("vanilla","obscurity-offset"))+"</anchor>");
            out.println("      <time>"+Tools.prettyTime(e.getEventTime())+"</time>");
            out.println("      <reports>");
            List l = e.getReports();
            Iterator i2 = l.iterator();
            while (i2.hasNext()) {
               Report r = (Report) i2.next();
               if (Debug.debug) Debug.print(NewsPage.class,"Generating Report "+r);
               Assassin tx = r.getAssassin();
               Map ecm = (Map) colourMaps.get(e);
               String colour = null;
               if (null != e)
               {
                  colour = (String) ecm.get(tx);
               }
               if (null == colour) colour = "nonparticipant";
               out.println("        <report colour=\"" + colour + "\">");
               Pseudonym px = r.getPseudonym();
               parttoRepl = updatePseudonymMap(parttoRepl, new Integer(0), tx, px, e);
               String thisReport=Tools.replace(r.getText(),parttoRepl);
               if (null != tx)
               {
                  out.println("          <pseudonym>"+Tools.replace("[p0]",parttoRepl)+"</pseudonym>");
               }
               out.println("          <text>"+thisReport+"</text>");
               out.println("        </report>");
            }
            out.println("      </reports>");
            out.println("    </event>");
         //}
         }
         out.println("  </events>");
      out.println("  <links>");
      for (Iterator i = getAllNewsPages().iterator(); i.hasNext();) {
      	NewsPage np = (NewsPage)i.next();
      	String l = np.getLinkName();
      	String t = np.getTitle();
      	if (l!=getLinkName()) out.println("   <a href=\""+l+"\">"+t+"</a>");
      }
      out.println("    <a href=\""+Config.getConfig("vanilla","updates-page-name")+".html\">Updates</a>");
      out.println("    <a href=\""+Config.getConfig("vanilla","headlines-page-name")+".html\">Headlines</a>");
      int index=1;
      String s=null;
      while (null!=(s=Config.getConfig("vanilla","template-page-name-"+index))) {
      	out.println("    <a href=\""+s+".html\">"+s+"</a>");
         index++;
      }
      out.println("  </links>");
      out.println("</webpage>");
      out.close();
      String xslt_template = Config.getConfig("vanilla","xsl-file");
      if (Debug.debug) Debug.print(this, "exec(new String[] { '/usr/bin/xsltproc', '-o', '"+
            base+".html', '"+xslt_template+"', '"+target+"'})");
      Tools.exec(new String[] {
         "/usr/bin/xsltproc",
         "--novalid",
         "-o",
         base+".html",
         xslt_template,
         target});
   }

   /**
 * @return
 */
public String getTitle() {
	return title;
}

private static class NewsComparator implements java.util.Comparator {

      public NewsComparator() {}

      public int compare(Object o, Object p) {
         long e=0, f=0;
         try {
            e = ((Event) o).getEventTime();
            f = ((Event) p).getEventTime();
         } catch (AssassinSQLException asqle) {
         }
         if (e<f) return -1;
         if (e==f) return 0;
         return 1;
      }
   }
}
