/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.TreeSet;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;
import org.ucam.srcf.assassins.xml.Config;

public class StateBuilder
{
   private Map plugins;
   private Set newspages=new TreeSet();
   
   public StateBuilder(Map p)
   {
      plugins = p;
   }
     
   Map doNews(long time) throws StateBuilderException
   {
      try
      {
         if (Debug.debug) Debug.print(StateBuilder.class, "doNews");
         Player.getAllPlayers();
         List assassins = Assassin.getAllAssassins();
         Player.associatePlayersWithAssassins();
         Map states = new HashMap();
         Iterator i = assassins.iterator();
         while (i.hasNext())
         {
            Assassin a = (Assassin) i.next();
            AssassinState s = new AssassinState(a);
            states.put(a, s);
         }
         NewsCreator nc = null;
         if (Debug.debug) Debug.print(StateBuilder.class, "instantiating NewsCreator");
         try {
            nc = new NewsCreator(Config.getConfig("vanilla","path-to-pages"), plugins);
         } catch (java.io.IOException ioe) {
            throw new StateBuilderException(ioe);
         }
         if (Debug.debug) Debug.print(StateBuilder.class, "instantiated ok");
         List events = Event.getEventsInTime(0, time);
         Event.preCacheSubClasses(0, time);
         // IMPORTANT: I'm assuming here that getAllEvents() returns them
         // in chronological order! otherwise this will fail horribly!
         i = events.iterator();
         if (Debug.debug) Debug.print(StateBuilder.class, "applying events");
         while (i.hasNext())
         {
            Event e = (Event) i.next();
            if (Debug.debug) Debug.print(StateBuilder.class, e.toString());
            if (e.isActive())
            {
               List sclist = e.getStateChanges();
               Iterator sci = sclist.iterator();
               while (sci.hasNext())
               {
                  StateChange sc = (StateChange) sci.next();
                  Assassin a = sc.getAssassin();
                  String pname = sc.getPluginName();
                  String field = sc.getFieldName();
                  Object value = sc.getValue();
                  if (Debug.debug) Debug.print(StateBuilder.class, "StateChange: "+sc);
                  // special cases
                  if (sc instanceof PseudonymGain)
                  {
                     if (Debug.debug) Debug.print(StateBuilder.class, "Pseudonym: "+ value.getClass());
                     if (Debug.debug) Debug.print(StateBuilder.class, "AssassinState: "+(AssassinState) states.get(a));
                     ((AssassinState)states.get(a)).addPseudonym((Pseudonym) value);
                     if (Debug.debug) Debug.print(StateBuilder.class, "Done");
                  }
                  else if (sc instanceof PseudonymLoss)
                  {
                     ((AssassinState)states.get(a)).removePseudonym((Pseudonym) value);
                  }
                  else
                  {
                     if (sc instanceof Death)
                     {
                        field = "alive";
                        value = "false";
                     }
                     else if (sc instanceof Resurrection)
                     {
                        field = "alive";
                        value = "true";
                     }
                     ((AssassinState)states.get(a)).setProperty(pname,field,value);
                  }
               }
               if (Debug.debug) Debug.print(StateBuilder.class, "Beginning event pass to NewsCreator");
               if (e.isVisible())
               {
                  Map part = new HashMap();
                  Iterator pi = ((e.getParticipants()).values()).iterator();
                  if (Debug.debug) Debug.print(StateBuilder.class, "About to start iterating");
                  while (pi.hasNext())
                  {
                     Assassin p = (Assassin) pi.next();
                     AssassinState ps = (AssassinState) states.get(p);
                     // Over Christmas 2003, mac57 assured amgb2 that a
                     // method AssassinState.deepClone() would never be
                     // needed. Less than two days before the start of the
                     // Michaelmas 2004 game, while frantically trying to
                     // fix a bug in mac57's code, amgb2 discovered that he
                     // had been wrong.
                     if (null == ps)
                     {
                        part.put(p, null);
                     }
                     else
                     {
                        part.put(p, ps.deepClone());
                     }
                  }
                  if (Debug.debug) Debug.print(StateBuilder.class, "done iterating");
                  if (Debug.debug) Debug.print(StateBuilder.class, "about to add event to NewsCreator");
                  nc.addEvent(e, part);
               }
               if (Debug.debug) Debug.print(StateBuilder.class, "Events passed");
            }
         }
         if (Debug.debug) Debug.print(StateBuilder.class, "generating news pages");
         nc.generate();
         newspages = nc.getPages();			
         return states;
      }
      catch (AssassinSQLException ASe)
      {
         throw new StateBuilderException(ASe);
      }
      catch (ParseException Pe)
      {
         throw new StateBuilderException(Pe);
      }
   }
  
   Map buildStateAtTime(long time) throws StateBuilderException
   {
      try
      {
         if (Debug.debug) Debug.print(StateBuilder.class, "buildStateAtTime");
         Player.getAllPlayers();
         List assassins = Assassin.getAllAssassins();
         Player.associatePlayersWithAssassins();
         Map states = new HashMap();
         Iterator i = assassins.iterator();
         while (i.hasNext())
         {
            Assassin a = (Assassin) i.next();
            AssassinState s = new AssassinState(a);
            states.put(a, s);
            if (Debug.debug) Debug.print(StateBuilder.class, "Creating State for "+a);
         }
         
         Pseudonym.preCache();
         StateChange.preCache();
         
         // do pseudonymns
         List sclist = StateChange.getPseudonymsAtTime(time);
         Iterator sci = sclist.iterator();
         while (sci.hasNext())
         {
            StateChange sc = (StateChange) sci.next();
            Assassin a = sc.getAssassin();
            Pseudonym p = (Pseudonym) sc.getValue();
            if (sc instanceof PseudonymGain)
            {
               if (Debug.debug) {
                  if (null != p) Debug.print(StateBuilder.class, "Pseudonym: "+ p.getClass());
                  else Debug.print(StateBuilder.class, "Pseudonym null in pseudonym gain. Surely this is bad?");
               }
               if (Debug.debug) Debug.print(StateBuilder.class, "Assassin: "+a);
               if (Debug.debug) Debug.print(StateBuilder.class, "AssassinState: "+(AssassinState) states.get(a));
               ((AssassinState)states.get(a)).addPseudonym(p);
               if (Debug.debug) Debug.print(StateBuilder.class, "Done");
            }
            else if (sc instanceof PseudonymLoss)
            {
               ((AssassinState)states.get(a)).removePseudonym(p);
            }
         }

         // do deaths/ressurrections
         List l1 = StateChange.getEveryonesFieldValueAtTime("vanilla", "killed-by", time);
         List l2 = StateChange.getEveryonesFieldValueAtTime("vanilla", "resurrected", time);
         Iterator i1 = l1.iterator();
         Iterator i2 = l2.iterator();
         Map rs = new HashMap();
         while (i2.hasNext()) {
            Resurrection r = (Resurrection) i2.next();
            rs.put(r.getAssassin(), new Long(r.getEvent().getEventTime()));
         }
         while (i1.hasNext()) {
            Death d = (Death) i1.next();
            Assassin a = d.getAssassin();
            Long rtime = (Long) rs.get(a);
            if (null == rtime)
               // they're dead, dave
               ((AssassinState)states.get(a)).setProperty("vanilla","alive","false");
            else if (rtime.longValue() < d.getEvent().getEventTime())
               // everybody's dead dave
               ((AssassinState)states.get(a)).setProperty("vanilla","alive","false");
            else
               // what, even captain hollister?!
               ((AssassinState)states.get(a)).setProperty("vanilla","alive","true");
         }

         // do other
         sclist = StateChange.getFieldValuesAtTime(time);
         sci = sclist.iterator();
         while (sci.hasNext())
         {
            StateChange sc = (StateChange) sci.next();
            Assassin a = sc.getAssassin();
            String pname = sc.getPluginName();
            String field = sc.getFieldName();
            Object value = sc.getValue();
            if (Debug.debug) Debug.print(StateBuilder.class, "StateChange: "+sc);
            // ignore special cases ones!
            if (sc instanceof PseudonymGain) ;
            else if (sc instanceof PseudonymLoss) ;
            else if (sc instanceof Death) ;
            else if (sc instanceof Resurrection) ;
            else
               ((AssassinState)states.get(a)).setProperty(pname,field,value);
         }

         // return it
         return states;
      }
      catch (AssassinSQLException ASe)
      {
         throw new StateBuilderException(ASe);
      }
   }
  
   Map reBuildStateAtTime(long oldtime, Map oldstate, long time) throws StateBuilderException
   {
      try
      {
         if (Debug.debug) Debug.print(StateBuilder.class, "reBuildStateAtTime");
         Map states = oldstate;
         List sclist = StateChange.getActiveBetweenTimes(oldtime, time);
         Iterator sci = sclist.iterator();
         while (sci.hasNext())
         {
            StateChange sc = (StateChange) sci.next();
            Assassin a = sc.getAssassin();
            String pname = sc.getPluginName();
            String field = sc.getFieldName();
            Object value = sc.getValue();
            if (Debug.debug) Debug.print(StateBuilder.class, "StateChange: "+sc);
            AssassinState ast = (AssassinState)states.get(a);
            if (null == ast) {
               ast = new AssassinState(a);
               states.put(a, ast);
            }
            // special cases
            if (sc instanceof PseudonymGain)
            {
               if (Debug.debug) Debug.print(StateBuilder.class, "Pseudonym: "+ value.getClass());
               if (Debug.debug) Debug.print(StateBuilder.class, "AssassinState: "+(AssassinState) states.get(a));
               ast.addPseudonym((Pseudonym) value);
               if (Debug.debug) Debug.print(StateBuilder.class, "Done");
            }
            else if (sc instanceof PseudonymLoss)
            {
               ast.removePseudonym((Pseudonym) value);
            }
            else
            {
               if (sc instanceof Death)
               {
                  field = "alive";
                  value = "false";
               }
               else if (sc instanceof Resurrection)
               {
                  field = "alive";
                  value = "true";
               }
               ast.setProperty(pname,field,value);
            }
         }
         return states;
      }
      catch (AssassinSQLException ASe)
      {
         throw new StateBuilderException(ASe);
      }
   }
   
   public Set getPages() {
   	return newspages;
   }
}
