/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */
package org.ucam.srcf.assassins;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Map;

public class Tools
{   
   public static final String NAME = "CUAG AutoUmpire";
   public static final String VERSION = "0.2";
   public static final String RELEASENAME = "Cone";
   public static final String VERSIONSTRING = NAME + " v"+VERSION + " ("+RELEASENAME+")";
 
   static DateFormat df = new SimpleDateFormat("d-M-yyyy H:mm:ss");
   static DateFormat fTim = new SimpleDateFormat("d MMM, HH:mm a");
   static DateFormat fDay = new SimpleDateFormat("EEEE, d MMMM");
   static DateFormat fToD = new SimpleDateFormat("HH:mm a");

   private static long nowtime = 0;
   public static long getNowTime() { return nowtime; }
   static void setNowTime(long time) { nowtime = time; }
   static void setNowTime(Date time) { nowtime = time.getTime(); }
   static void setNowTime(String time) throws ParseException { nowtime = unFormatTime(time); }
   
   public static long unFormatTime(String timestr) throws ParseException
   {
      /*StringTokenizer tokenizer = new StringTokenizer(timestr,":- ");
      Date x;
      try {
         int day = Integer.parseInt(tokenizer.nextToken());
         int month=Integer.parseInt(tokenizer.nextToken());
         int year= Integer.parseInt(tokenizer.nextToken());
         if (year<30) year +=100;
         int hour= Integer.parseInt(tokenizer.nextToken());
         int minute=Integer.parseInt(tokenizer.nextToken());
         int second=Integer.parseInt(tokenizer.nextToken());
         if (Debug.debug) Debug.print(Tools.class, "Parsed date" + year + " " + month + " " + day + " " + hour + " " + minute + " " + second);
       x = new Date(year,month,day,hour,minute,second);
      } catch (Exception e) {
         return -1;
      }*/
      SimpleDateFormat d = new SimpleDateFormat("d-M-yyyy H:mm:ss");
      if (Debug.debug) Debug.print(Tools.class, "Parsing " + timestr);
      Date x = d.parse(timestr);
      //      if (Debug.debug) Debug.print(Tools.class, "Parsed as "+x.toGMTString()); 
      return x.getTime();
   }

   public static String formatTime(long time)
   {
      return df.format(new Date(time));
   }

   public static String prettyDateAndTime(long time)
   {
      return fTim.format(new Date(time));
   }
   
   public static String prettyDate(long time)
   {
      return fDay.format(new Date(time));
   }
   
   public static String prettyTime(long time)
   {
      return fToD.format(new Date(time));
   }

   public static String escape(String in)
   {
      if (Debug.debug) Debug.print(Tools.class, "escaping: "+in);
      String out = "";
      for (int i = 0; i < in.length(); i++) {
         if (in.charAt(i) == '\'') out += '\\';
         out += in.charAt(i);
      }
      return out;
   }

   public static String escapeXML(String in)
   {
      if (in == null) return null;
      String out = "";
      for (int i = 0; i < in.length(); i++)
      {
         switch (in.charAt(i))
         {
            case '&':
               // preserve existing entities
               if( i+1 == in.length()) { out += "&amp;"; break; }
               String s = "";
               i++;
               while ( i < in.length() && "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz".indexOf(in.charAt(i)) > 0 ) 
                  s += in.charAt(i++);
               if (i == in.length()) out += "&amp;" + s;
               else if (';' == in.charAt(i)) out += "&"+s+";";
               else out += "&amp;" + s + in.charAt(i);
               break;
            case '<':
               out += "&lt;"; break;
            case '>':
               out += "&gt;"; break;
            case '"':
               out += "&quot;"; break;
            default:
               out += in.charAt(i);
         }
      }
      return out;
   }
   
   public static void exec(String[] args)
   {
      try {
         Process p = Runtime.getRuntime().exec(args);
         BufferedReader stderr = new BufferedReader(new InputStreamReader(p.getErrorStream()));
         String s;
         while (null != (s = stderr.readLine()))
            System.err.println(s);
         p.waitFor();
      } 
      catch (InterruptedException Ie) { Ie.printStackTrace(); }
      catch (IOException IOe) { IOe.printStackTrace(); }
   }
 
   public static String replace(String x, Map withBy) {
      for (int i=0; i<x.length(); i++) {
         if (x.charAt(i)!='[') continue;
         String y = "";
         char t;
         int j =0;
         while ((t = x.charAt(i+(++j)))!=-1 && ']'!=t) y+=t;
         String repl = (String) ((null==withBy.get(y))?"["+y+"]":withBy.get(y));
         x = x.substring(0,i)+repl+x.substring(i+j+1, x.length());
         i+=repl.length();
      }
      return x;
   }

}
