/* 
 * CUAG AutoUmpire System
 * 
 * Copyright (C) 2003/2004 Matthew Johnson, Adam Biltcliffe,
 * Michael Cripps, Martin O'Leary, Edward Allcutt and James Osborn
 *
 * This program is free software; you can redistribute it and/or 
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details. 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * To Contact the authors, please email assassins@srcf.ucam.org
 *
 */

package org.ucam.srcf.assassins.stracciatella;

import java.text.ParseException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;
import java.util.Vector;

import cx.ath.matthew.cgi.CGI;

import org.ucam.srcf.assassins.AssassinStateViewer;
import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.Death;
import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.Event;
import org.ucam.srcf.assassins.EventViewer;
import org.ucam.srcf.assassins.Plugin;
import org.ucam.srcf.assassins.PluginCGICommand;
import org.ucam.srcf.assassins.PluginCommand;
import org.ucam.srcf.assassins.PluginStateChange;
import org.ucam.srcf.assassins.StateChange;
import org.ucam.srcf.assassins.YesNoQuestion;
import org.ucam.srcf.assassins.exceptions.AssassinSQLException;
import org.ucam.srcf.assassins.exceptions.StateBuilderException;

/**
 * Interface for java classes which act as plug-in game modifiers.
 */
public class Stracciatella extends Plugin
{
   public static String getDescription() { return "Stats building module (kills, conkers, position)"; };
   /**
    * Called by the filtering mechanism when the plugin needs to determine
    * whether an assassin has a given property such as &quot;police&quot;. If
    * the plugin does not recognise the named property, it should return
    * <code>false</code>.
    * @param a A viewer for the assassin being tested.
    * @param prop The name of the property being asked about.
    * @return True if the assassin has the property, false if not.
    */
   public boolean assassinHasProperty(AssassinStateViewer a, String prop, String value)  throws AssassinSQLException
   {
      if ("hasstats".equals(prop)) {
         String v = a.getProperty(getPluginName(), "hasstats");
         return value.equals(v);
      }
      else return false;
   }

   /**
    * Called when an assassin is being added to get any queries the plugin
    * has for the user. The plugin should check all the assassin details and if it
    * needs more information it should ask relevent questions. The 
    * returnQuestions function is passed the same Vector with the answers 
    * filled out.
    * @param a The Assassin which is being added
    * @return A Vector of Questions
	 * @param state a Map AssassinViewer=&gt;AssassinStateViewer
    * @see returnQuestions
    */
   protected Vector getQuestions(AssassinViewer a, Map state) throws AssassinSQLException 
   {
      Vector qs = new Vector();
      qs.add(new YesNoQuestion("Track Stats",true));
      return qs;
   }
   
   /**
    * Returns the Questions with the answers filled out
    *
    * @param a The Assassin being added
    * @param e An Event to add StateChanges to
	 * @param state a Map AssassinViewer=&gt;AssassinStateViewer
    * @param qs The Questions which were asked
	 * @see getQuestions
    */
   protected void returnQuestions(AssassinViewer a, EventViewer e, Map state, Vector qs) throws AssassinSQLException 
   {
      boolean dostats = ((YesNoQuestion)qs.get(0)).getAnswer();
      vanilla.addStateChange(e, new PluginStateChange(a, this, "hasstats", dostats?"true":"false"));
   }

   public void updateStats(Map params) throws AssassinSQLException, StateBuilderException
   {
      // create maps of data
      Map kills = new HashMap();
      Map rank = new HashMap();
      Map conkers = new HashMap();
      Map state = vanilla.getState();
      Iterator i = state.keySet().iterator();
      Integer zero = new Integer(0);
      Integer one = new Integer(1);
      int live = 0;
      while (i.hasNext()) {
         AssassinViewer a = (AssassinViewer) i.next();
         if ("true".equals(((AssassinStateViewer) state.get(a)).getProperty(getPluginName(), "hasstats"))) {
            if (Debug.debug) Debug.print(this, "Doing stats for "+a.getPrintable());
            kills.put(a, zero);
            conkers.put(a, zero);
            rank.put(a, one);
            if (((AssassinStateViewer) state.get(a)).isAlive()) live++;
         }
      }
      
      // get kills
      List scs = StateChange.getByClassAtTime(Death.class,vanilla.getNowTime());
      i = scs.iterator();
      while (i.hasNext()) {
         Death d = (Death) i.next();
         if (kills.get(d.getKiller()) == null) continue;
         if (Debug.debug) Debug.print(this, "Kill for "+d.getKiller().getPrintable());
         kills.put(d.getKiller(), new Integer((((Integer) kills.get(d.getKiller())).intValue() + 1)));
      }

      // get conkers
      i = scs.iterator();
      while (i.hasNext()) {
         Death d = (Death) i.next();
         if (kills.get(d.getKiller()) == null) continue;
         if (Debug.debug) Debug.print(this, "Conkers Update for "+d.getKiller().getPrintable());
         Integer vkills = (Integer) kills.get(d.getVictim());
         Integer kconk = (Integer) conkers.get(d.getKiller());
         if (null == vkills) vkills = zero;
         conkers.put(d.getKiller(), 
               new Integer(vkills.intValue() + kconk.intValue() + 1));
      }

      // get rank
      ListIterator li = scs.listIterator(scs.size());
      Integer crank = new Integer(live);
      long lasttime = Long.MAX_VALUE;
      int inc = 0;
      while (li.hasPrevious()) {
         Death d = (Death) li.previous();
         if (rank.get(d.getVictim()) == null) continue;
         if (d.getEventViewer().getTime() < lasttime) {
            crank = new Integer(crank.intValue() + inc + 1);
            inc = 0;
         }
         else inc++;
         if (Debug.debug) Debug.print(this, "rank "+crank+" for "+d.getVictim().getPrintable());
         lasttime = d.getEventViewer().getTime();
         rank.put(d.getVictim(), crank);
      }

      // set state values
      Event e = new Event();
      e.setEventTime(vanilla.getNowTime());
      e.setVisible(false);
      e.setActive(true);
      i = kills.keySet().iterator();
      while (i.hasNext()) {
         AssassinViewer a = (AssassinViewer) i.next();
         e.addStateChange(new PluginStateChange(a, this, "kills", kills.get(a).toString()));
         e.addStateChange(new PluginStateChange(a, this, "conkers", conkers.get(a).toString()));
         e.addStateChange(new PluginStateChange(a, this, "rank", rank.get(a).toString()));
      }
   }

   public void CGIupdateStats(CGI cgi, Map params) throws AssassinSQLException, StateBuilderException
   {
      cgi.out("<p>");
      cgi.out("Updating stats, please wait...");
      cgi.flush();
      updateStats(params);
      cgi.out("... Done.");
      cgi.out("</p>");
   }

   /**
    * Returns a Map of new commands to add to the CLI
    *
    * @return A Map of String =&gt; Command
    */
   public Map getCommands() 
   { 
      Map m = new HashMap();
      final Stracciatella strac = this;
      m.put("update-stats",
            new PluginCommand () {
               public void pluginCommand(Map params) throws AssassinSQLException, StateBuilderException
               {
                  strac.updateStats(params);
               }
            }
           );
      return m;
   }

   /**
    * Returns a Map of new commands to add to the CGI
    *
    * @return A Map of String =&gt; CGICommand
    */
   public Map getCGICommands() 
   {
      Map m = new HashMap();
      final Stracciatella strac = this;
      m.put("update-stats",
            new PluginCGICommand () {
               public void pluginCommand(CGI cgi, Map params) throws AssassinSQLException, StateBuilderException
               {
                  strac.CGIupdateStats(cgi, params);
               }
               public String getName() { return "Update Stats"; }
               public String getDescription() { return "Calculate stats for all the players and update the state with them"; }
               public boolean isPublic() { return false; }
            }
           );
      return m;
   }
}


