/**
 * Class to track changes in differnt variables for all players
 */

package org.ucam.srcf.assassins.strawberry;

import java.util.Map;
import java.util.Iterator;
import java.util.HashMap;

import org.ucam.srcf.assassins.AssassinViewer;
import org.ucam.srcf.assassins.Death;
import org.ucam.srcf.assassins.EventViewer;

import org.ucam.srcf.assassins.exceptions.AssassinSQLException;

class VarTracker {
   Map state;

   Map scores;

   String pluginName;

   VarTracker(Map state, String pname) throws AssassinSQLException {
      /* Generate an empty map */
      this.state = new HashMap();
      Iterator i = state.keySet().iterator();
      while (i.hasNext())
         this.state.put(i.next(), new HashMap());
      /* Setup a new set of scores */
      refreshScores(null);
      /* Cache the plugin name which owns us */
      this.pluginName = pname;
   }

   public Map getState() {
      return state;
   }

   public void setProperty(AssassinViewer a, String what, String value) {
      Map aMap = (Map) state.get(a);
      aMap.put(what, value);
   }

   public String getProperty(AssassinViewer a, String what) {
      Map aMap = (Map) state.get(a);
      return (String) aMap.get(a); // should be aMap.get(what);? -- MikeC
   }

   public void setIntProperty(AssassinViewer a, String what, int value) {
      setProperty(a, what, String.valueOf(value));
   }

   /**
    * returns the property cast to an integer.
    * 
    * @param a
    *           The Assassin the property belongs to.
    * @param what
    *           The value to retrieve
    * @return The value in question, 0 if it doens't exist
    */
   public int getIntProperty(AssassinViewer a, String what) {
      return Integer.parseInt(getProperty(a, what));
   }

   /**
    * Increment the given property by a specified amount.
    * 
    * @param a
    *           The Assassin the Property belongs to.
    * @param what
    *           String specifying what the property is.
    * @param increment
    *           The amount to increment it by.
    * @return The new value
    */
   public int incIntProperty(AssassinViewer a, String what, int increment) {
      int initial = getIntProperty(a, what);
      initial += increment;
      setIntProperty(a, what, initial);
      return initial;
   }

   public Map refreshScores(EventViewer e) throws AssassinSQLException {
      if (e == null) return scores = null;

      scores = new HashMap();
      Map participants = e.getParticipants();
      Iterator k = participants.keySet().iterator();
      while (k.hasNext())
         scores.put(k.next(), null);

      return scores;
   }

   public void trackInnocent(AssassinViewer a) {
      incIntProperty(a, "Innocents", 1);
   }

   public void trackDeath(Death sc) {
      AssassinViewer killer, victim;
      try {
         killer = sc.getKiller();
         victim = sc.getVictim();
      } catch (AssassinSQLException asqle) {
         return;
      }

      /* require victim to be alive! */
      if (getProperty(victim, "alive").equals("false")) return;

      incIntProperty(killer, "Kills", 1);
      incIntProperty(killer, "KillsSinceDeath", 1);
      incIntProperty(victim, "Deaths", 1);
      setIntProperty(victim, "KillsSinceDeath", 0);
   }
}