package org.ucam.srcf.assassins.xml;

// JAXP
//import javax.xml.parsers.*;
//import org.xml.sax.*;
//import org.xml.sax.helpers.*;
import java.util.HashMap;
import java.util.Map;

import org.ucam.srcf.assassins.Debug;
import org.ucam.srcf.assassins.exceptions.ConfigFileException;
import org.ucam.srcf.assassins.exceptions.XMLParseException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
//import org.ucam.srcf.assassins.exceptions.XMLParseException;

/**
 * Program to allow the autoumpire to have behaviour specified in an XML file. Use JAXP to
 * read in and create a DOM tree. Then as long as a reference is kept to this object, we
 * can perform lookups on it for any required attribute.
 *
 * @author Mike Cripps <mac57@srcf.ucam.org>
 */

public class Config {

   static XMLParser myXMLParser;
   static Map colourFilters;

   public static void init(String filename) throws ConfigFileException {
      try {
		myXMLParser = new XMLParser(filename);
	} catch (XMLParseException e) {
		throw new ConfigFileException(e.getMessage());
	}
   }

   public static NodeList getTagList(String tagname) {
      return myXMLParser.getTagList(tagname);
   }

   public static Map getPluginList() {
      return myXMLParser.getNodeList("plugin");
   }

   private static Node getPlugin(String plugin) {
      return myXMLParser.getNodeWithName("plugin","name",plugin);
   }

   public static String getConfig(String plugin, String parameter) {
      return myXMLParser.lookup(plugin, parameter);
   }

   public static int getConfigInt(String plugin, String parameter)
   {
      try
      {
         if (Debug.debug) Debug.print(Config.class, "getConfigInt, " + plugin + "," + parameter);
         int result;
         String tmp;
         tmp = getConfig(plugin, parameter);
         if (tmp == null) return 0;
         if (Debug.debug) Debug.print(Config.class, "entering mysteriously failing code with " + tmp);
         
         result = Integer.parseInt(tmp);
         if (Debug.debug) Debug.print(Config.class, "leaving mysteriously failing code");
         return result;
      }
      catch (NumberFormatException NFex)
      {
         return 0;
      }
   }

   public static Map getColourFilters()
   {
      if (Debug.debug) Debug.print(Config.class, "getColourFilters");
      if (null != colourFilters) return colourFilters;
      Map result = new HashMap();
      NodeList nl = getTagList("colourfilter");
      for (int i = 0; i<nl.getLength(); i++)
      {
         Element n = (Element) nl.item(i);
         if (Debug.debug)
         {
            ColourFilter cf = ColourFilter.parseElement(n);
            result.put(n.getAttribute("name"), cf);
            Debug.print(Config.class, cf.debugName());
         }
         else
         {
            result.put(n.getAttribute("name"), ColourFilter.parseElement(n));
         }
      }
      colourFilters = result;
      return result;
   }
}
